import os
import subprocess
from playwright.sync_api import sync_playwright

def md_to_revealjs_html(slide_md, theme: str = "white"):
    # Convert input markdown to pandoc reveal.js html
    result = subprocess.run([
        "pandoc", "-f", "markdown", "-t", "revealjs", "-s", "-V", f"theme={theme}"
    ], input=slide_md , capture_output=True, text=True, check=True)
    return result.stdout

def revealjs_html_to_slide_images(
    html_path: str,
    output_dir: str,
    width: int = 1920,
    height: int = 1080,
    output_pdf: bool = True,
    headless = True,
) -> None:
    """
    Convert a Markdown file into Reveal.js slides and capture each slide as an image using Playwright.

    Steps:
    1. Uses Pandoc to generate an HTML slideshow (Reveal.js) from the Markdown.
    2. Uses headless Chromium via Playwright to load the generated HTML.
    3. Iterates through each slide, takes a full-page screenshot, and saves it as PNG.

    Args:
        md_path: Path to the input Markdown file.
        output_dir: Directory where slide images will be saved.
        theme: Reveal.js theme (e.g., "white", "black", "solarized").
        width: Width of the viewport for screenshots.
        height: Height of the viewport for screenshots.

    Prerequisites:
        - pandoc installed and in PATH
        - playwright installed (`pip install playwright`) and browsers installed (`playwright install`)
        - Internet access if using the default CDN for Reveal.js
    """
    # Ensure output directory exists
    os.makedirs(output_dir, exist_ok=True)
    assert os.path.exists(html_path)

    # 2. Launch headless browser and capture slides
    with sync_playwright() as pw:
        browser = pw.chromium.launch(headless=headless)
        context = browser.new_context(viewport={"width": width, "height": height})
        page = context.new_page()

        # Load the generated slideshow
        page.goto(f"file://{os.path.abspath(html_path)}", wait_until="domcontentloaded")
        # Wait for Reveal.js to load and initialize
        page.wait_for_selector(".reveal")
        page.wait_for_function("() => typeof Reveal !== 'undefined' && Reveal.isReady()")

        # Count slides
        slide_count = page.evaluate("() => Reveal.getTotalSlides()")
        
        if output_pdf:
            page.pdf(path=os.path.join(output_dir, f"all.pdf"), print_background=True, width='1920px', height='1080px')

        # Capture each slide
        for i in range(1, slide_count+1):
            # Navigate to slide i
            if i != 1:
                page.evaluate(f"Reveal.next();")

            # Wait for transitions/layout
            page.wait_for_timeout(2000)
            out_path = os.path.join(output_dir, f"{i}.png")
            page.screenshot(path=out_path, full_page=True)
            # print(f"Saved slide {i} to {out_path}")


        if output_pdf:
            # page.emulate_media(media="screen")
            for i in range(1, slide_count+1):
                pdf_path = os.path.join(output_dir, f"{i}.pdf")
                page.pdf(page_ranges = f'{i}', path=pdf_path, print_background=True, width='1920px', height='1080px')

        # import time; time.sleep(999)
        browser.close()


if __name__ == "__main__":
    import argparse

    parser = argparse.ArgumentParser(
        description="Convert Markdown to Reveal.js slides and capture each as an image using Playwright."
    )
    parser.add_argument("md", help="Path to the Markdown file")
    parser.add_argument("outdir", help="Directory to save slide images")
    parser.add_argument("--theme", default="white", help="Reveal.js theme (e.g., white, black, league)")
    args = parser.parse_args()
    
    with open(args.md, 'r') as f:
        input_md_data = f.read()
    
    slide_html = md_to_revealjs_html(input_md_data)
    out_html = f'{args.md}.html'
    with open(out_html, 'w') as f:
        f.write(slide_html)
    revealjs_html_to_slide_images(out_html, args.outdir, theme=args.theme)



import subprocess
import json
from pathlib import Path
from functools import cache
from copy import copy
import os
import sys

script_path = os.path.realpath(__file__)
script_dir = os.path.dirname(script_path)
project_dir = os.path.dirname(script_dir)

@cache
def _empty_ast_val() -> dict:
    return md_to_ast('')

def empty_ast() -> dict:
    return copy(_empty_ast_val())

def wrap_ast(ast):
    val = empty_ast()
    assert type(ast) is list
    val['blocks'] = ast
    return val

def md_to_ast(input_md: str) -> dict:
    # Convert input markdown to pandoc JSON AST
    result = subprocess.run([
        "pandoc", "-f", "markdown", "-t", "json"
    ], input=input_md , capture_output=True, text=True, check=True)
    return json.loads(result.stdout)


def write_pandoc_json(doc: dict, output_path: Path):
    with output_path.open('w', encoding='utf-8') as f:
        json.dump(doc, f, ensure_ascii=False, indent=2)


def ast_to_markdown(json_path: Path, md_path: Path):
    subprocess.run([
        "pandoc", str(json_path), "-f", "json", "-t", "markdown"
    ], check=True)

def ast_to_markdown(val) -> str:
    """
    Convert a Pandoc JSON AST (as a JSON string) directly to Markdown string using subprocess pipes.

    Args:
        json_str: The Pandoc AST in JSON format as a string.
    Returns:
        The converted Markdown as a string.
    """
    if type(val) is dict or type(val) is list:
        if type(val) is list:
            val = wrap_ast(val)
        val = json.dumps(val, indent=2, ensure_ascii=False)
    assert type(val) is str, 'Wrong type!'
    try:
        result = subprocess.run(
            ["pandoc", "-f", "json", "-t", "markdown"],
            input=val,
            text=True,
            capture_output=True,
            check=True
        )
    except subprocess.CalledProcessError as e:
        print("Error: convert ast to markdown failed!")
        print(e.stderr)
        print(val)
        raise e
    return result.stdout

def text_to_speech(text, output_path, lang=None, speed=1.0):
    from .tts.kokoro import text_to_speech
    text_to_speech(text, output_path, lang, speed)



# A live server that returns rendered reveal.js html

import os, io, sys, json
from http.server import SimpleHTTPRequestHandler, test, ThreadingHTTPServer, HTTPStatus
from .md2slidemd import md_to_slide_md, md_to_ast
from .md2image import md_to_revealjs_html

class SlideMDHandler(SimpleHTTPRequestHandler):
    '''
    returns rendered reveal.js html
    '''
    server_version = "SlideMDHandler/1.0"
    index_pages = ("index.html", "index.htm")
    extensions_map = _encodings_map_default = {
        '.gz': 'application/gzip',
        '.Z': 'application/octet-stream',
        '.bz2': 'application/x-bzip2',
        '.xz': 'application/x-xz',
    }
    show_slide_md=False

    def do_GET(self):
        """Serve a GET request."""
        path = self.translate_path(self.path)
        if path.endswith('.md') and os.path.isfile(path):
            print(f"Generating reveal.js result for {path}")
            with open(path, 'r') as md_file:
                data = md_file.read()
                if self.show_slide_md:
                    slide_md = md_to_slide_md(data)
                    import html
                    slide_md = f'<div style="white-space: pre-wrap;">{html.escape(slide_md)}</div>'
                else:
                    # slide_md = json.dumps(md_to_ast(md_to_slide_md(data)), ensure_ascii=False)
                    slide_md = md_to_revealjs_html(md_to_slide_md(data))
                    # slide_md = md_to_revealjs_html(data)
            enc = sys.getfilesystemencoding()
            encoded = slide_md.encode(enc, 'surrogateescape')
            f = io.BytesIO()
            f.write(encoded)
            f.seek(0)
            self.send_response(HTTPStatus.OK)
            self.send_header("Content-type", "text/html; charset=%s" % enc)
            self.send_header("Content-Length", str(len(encoded)))
            self.end_headers()
            try:
                self.copyfile(f, self.wfile)
            finally:
                f.close()
        else:
            f = self.send_head()
            if f:
                try:
                    self.copyfile(f, self.wfile)
                finally:
                    f.close()


if __name__ == '__main__':
    import argparse
    parser = argparse.ArgumentParser()
    parser.add_argument('-b', '--bind', default='127.0.0.1', metavar='ADDRESS',
                        help='bind to this address '
                             '(default: all interfaces)')
    parser.add_argument('-d', '--directory', default=os.getcwd(),
                        help='serve this directory '
                             '(default: current directory)')
    parser.add_argument('-p', '--protocol', metavar='VERSION',
                        default='HTTP/1.0',
                        help='conform to this HTTP version '
                             '(default: %(default)s)')
    parser.add_argument('port', default=8000, type=int, nargs='?',
                        help='bind to this port '
                             '(default: %(default)s)')
    parser.add_argument('--show-slide-md', action='store_true', default=False)

    args = parser.parse_args()
    if args.show_slide_md:
        SlideMDHandler.show_slide_md = True
    HandlerClass = SlideMDHandler
    test(
        HandlerClass=SlideMDHandler,
        ServerClass=ThreadingHTTPServer,
        port=args.port,
        bind=args.bind,
        protocol=args.protocol,
    )
    # ServerClass = ThreadingHTTPServer
    # import socket
    # def _get_best_family(*address):
    #     infos = socket.getaddrinfo(
    #         *address,
    #         type=socket.SOCK_STREAM,
    #         flags=socket.AI_PASSIVE,
    #     )
    #     family, type, proto, canonname, sockaddr = next(iter(infos))
    #     return family, sockaddr
    # ServerClass.address_family, addr = _get_best_family(args.bind, args.port)
    # HandlerClass.protocol_version = args.protocol
    # with ServerClass(addr, HandlerClass) as httpd:
    #     host, port = httpd.socket.getsockname()[:2]
    #     url_host = f'[{host}]' if ':' in host else host
    #     print(
    #         f"Serving HTTP on {host} port {port} "
    #         f"(http://{url_host}:{port}/) ..."
    #     )
    #     try:
    #         httpd.serve_forever()
    #     except KeyboardInterrupt:
    #         print("\nKeyboard interrupt received, exiting.")
    #         sys.exit(0)




import sys,os
from pathlib import Path
from .md2slidemd import md_to_ast, md_to_slides
from .utils import *


def main():
    if len(sys.argv) != 3:
        print(f"Usage: python {sys.argv[0]} input.md output.mkv")
        sys.exit(1)

    input_md = Path(sys.argv[1])
    tmp_folder = Path(sys.argv[1]+'.md2video')
    os.makedirs(tmp_folder, exist_ok=True)
    out_video = Path(sys.argv[2])

    assert str(input_md).endswith('.md')
    assert str(out_video).endswith('.mkv')

    with open(input_md, 'r') as f:
        input_md_data = f.read()

    slides = md_to_slides(input_md_data)
    slides.set_workdir(tmp_folder)
    slides.create_video('out.mkv')

if __name__ == '__main__':
    main()



from .utils import *
from .md2image import *
from ffmpeg import FFmpeg, Progress
from ffmpeg.errors import FFmpegError

class Slide:
    def __init__(self, md_text):
        self.md_text = md_text
        self.parent = None
        self.base_path = None
        self.comments = None
        self.video_clip_path = None
    def get_lang(self):
        return self.parent.lang
    def set_parent(self, slides):
        self.parent = slides
    def set_base_path(self, base_path):
        self.base_path = base_path
    def set_comments(self, comments):
        self.comments = comments
    def create_audio(self, output_path):
        print("[!] Generating audios...")
        text_to_speech(self.comments, output_path, lang=self.get_lang())
        assert os.path.exists(output_path)
    def get_image_path(self):
        image_path = self.base_path + '.png'
        if os.path.exists(image_path):
            return image_path
        self.parent._create_images()
        assert os.path.exists(image_path)
        return image_path
    def _get_workdir(self):
        return self.parent.workdir
    def get_audio_path(self):
        assert self.base_path is not None
        audio_path = self.base_path + '.wav'
        if not os.path.exists(audio_path):
            self.create_audio(audio_path)
        return audio_path

    def create_video(self, out_path):
        try:
            # Create a video from image and audio using ffmpeg-python
            ffmpeg = FFmpeg() \
                .input(str(self.get_image_path()),loop=1) \
                .input(str(self.get_audio_path())) \
                .output(str(out_path), acodec='aac', vcodec='libx264',
                        framerate='30', pix_fmt='yuv420p', shortest=None,
                        s='1920x1080')

            @ffmpeg.on("progress")
            def on_progress(progress: Progress):
                print(progress)
            
            ffmpeg.execute()
            assert os.path.exists(out_path)
        except FFmpegError as e:
            print(e.message)

    def get_video_path(self):
        if self.video_clip_path is not None:
            return self.video_clip_path
        self.video_clip_path = self.base_path + '.mkv'
        if not os.path.exists(self.video_clip_path):
            self.create_video(self.video_clip_path)
        return self.video_clip_path


class Slides:
    def __init__(self, slides = []):
        self.data = slides # type: list[Slide]
        for s in slides:
            s.set_parent(self)
        self.workdir = None
        self.slide_html_path = None
        self.lang = None

    def set_lang(self, lang):
        self.lang = lang

    def set_workdir(self, workdir):
        self.workdir = workdir
        for i, s in enumerate(self.data):
            s.set_base_path(os.path.join(workdir, str(i + 1)))

    def to_slide_md_text(self):
        slides = self.data
        if len(slides) == 0:
            return []
        blocks = []
        for slide in slides:
            blocks.append(slide.md_text)
            if slide.comments is not None:
                blocks += [f'''::: notes
{slide.comments}
:::''']
            blocks += ['------------------------------------------------------------------------']
        blocks = blocks[:-1]
        return '\n\n'.join(blocks)

    def get_slide_html_path(self):
        if self.slide_html_path is not None:
            return self.slide_html_path
        slide_html = md_to_revealjs_html(self.to_slide_md_text())
        self.slide_html_path = os.path.join(self.workdir, "reveal.html")
        with open(self.slide_html_path, 'w') as f:
            f.write(slide_html)
        return self.slide_html_path

    def _create_images(self):
        print("[!] Generating images from slides...")
        revealjs_html_to_slide_images(self.get_slide_html_path(), self.workdir)

    def create_video(self, out_path):
        '''
        Uses ffmpeg filter_complex to concatenate videos, handling different video parameters
        https://stackoverflow.com/a/11175851
        '''
        print("[!] Concat videos...")
        if not os.path.exists(out_path):
            videos = []
            for slide in self.data:
                videos.append(slide.get_video_path())
            
            # Build filter complex string
            n_videos = len(videos)
            filter_complex = []
            
            # Add input arguments and build filter parts
            for i in range(n_videos):
                filter_complex.append(f'[{i}:v] [{i}:a]')

            filter_str = ' '.join(filter_complex)
            filter_str += f' concat=n={n_videos}:v=1:a=1 [v] [a]'

            # try:
            # Configure ffmpeg
            ffmpeg = FFmpeg()
            for i in range(n_videos):
                ffmpeg.input(str(videos[i]))
            ffmpeg.output(
                str(out_path),
                filter_complex=filter_str,
                map=['[v]', '[a]'],
            )

            @ffmpeg.on("progress")
            def on_progress(progress: Progress):
                print(progress)
                
            print(' '.join(ffmpeg.arguments))
            ffmpeg.execute()
            assert os.path.exists(out_path)
            # except FFmpegError as e:
            #     print(ffmpeg._process.stderr)
            #     raise e
        return out_path

    # def create_video(self, out_path):
    #     '''
    #     https://github.com/kkroening/ffmpeg-python/issues/96#issuecomment-401530613
    #     '''
    #     print("[!] Concat videos...")
    #     if not os.path.exists(out_path):
    #         videos = []
    #         for slide in self.data:
    #             videos.append(slide.get_video_path())
    #         # 用ffmpeg合并所有的video
    #         ffmpeg = FFmpeg()
    #         ffmpeg.input(f'concat:{"|".join(videos)}').output(str(out_path), c='copy')

    #         @ffmpeg.on("progress")
    #         def on_progress(progress: Progress):
    #             print(progress)
    #         ffmpeg.execute()
    #         assert os.path.exists(out_path)
    #     return out_path



# AST transform to slide js.
import sys
from pathlib import Path
from .utils import *
from .slide import *
import re


def remove_html_comments(s: str):
    if s.startswith('<!--'):
        if s.endswith('-->'):
            return s.removeprefix('<!--').removesuffix('-->')
    return None


# @deprecated
def md_to_slides(md_text) -> Slides:
    state = None
    buffer = []
    slides = [] # type: list[Slide]

    def finish_prev_state():
        nonlocal buffer
        nonlocal slides
        nonlocal state
        if state is None or state == 'empty' or state == 'none':
            pass
        elif state == 'slide':
            slides.append(Slide('\n\n'.join(buffer)))
        elif state == 'comment':
            slides[-1].set_comments('\n\n'.join(buffer))
        else:
            assert False, 'Unknown state!'
        buffer = []
        state = None

    while len(md_text := md_text.strip()) > 0:
        m = re.search(r'<!--\s*mode:?\s*(\w+?)\s*-->', md_text)
        next_start = m.start() if m is not None else len(md_text)
        if next_start != 0:
            buffer.append(md_text[:next_start].strip())
            md_text = md_text[next_start:]
            continue
        mode_text = m.group(1)
        if mode_text == 'slide':
            finish_prev_state()
            state = 'slide'
            buffer = []
        elif mode_text == 'comment':
            finish_prev_state()
            state = 'comment'
        elif mode_text == 'empty':
            finish_prev_state()
            state = 'empty'
        else:
            assert False, f"Unknown mode: {mode_text}!"
        md_text = md_text[m.end():]
        continue
    return Slides(slides)

def md_to_slide_md(input_md):
    # Step2: process AST
    slides = md_to_slides(input_md)
    new_md = slides.to_slide_md_text()
    return new_md

def main():
    if len(sys.argv) != 3:
        print(f"Usage: python {sys.argv[0]} input.md output.md")
        sys.exit(1)

    input_md = Path(sys.argv[1])
    out_md = Path(sys.argv[2])
    
    assert str(input_md).endswith('.md'), "input file extension is not .md!"

    with open(input_md, 'r') as f:
        input_md_data = f.read()

    new_md = md_to_slide_md(input_md_data)

    with open(out_md, 'w') as f:
        f.write(new_md)


if __name__ == '__main__':
    main()




import os
from misaki import zh, en
from kokoro_onnx import Kokoro
import soundfile as sf

script_path = os.path.realpath(__file__)
script_dir = os.path.dirname(script_path)
project_dir = os.path.dirname(os.path.dirname(script_dir))

# https://github.com/thewh1teagle/kokoro-onnx/blob/main/examples/chinese.py
DOWN_FILE = '''Download these files
https://github.com/thewh1teagle/kokoro-onnx/releases/download/model-files-v1.1/kokoro-v1.1-zh.onnx
https://github.com/thewh1teagle/kokoro-onnx/releases/download/model-files-v1.1/voices-v1.1-zh.bin
https://huggingface.co/hexgrad/Kokoro-82M-v1.1-zh/raw/main/config.json'''


# default_voice_for_lang = {
#     'en-us': 'af_nicole',
#     'cmn': 'zm_yunjian',
# }

# Misaki G2P with espeak-ng fallback
en_g2p = en.G2P(trf=False, british=False)
def en_callable(text: str) -> str:
    return en_g2p(text)[0]
g2p = zh.ZHG2P(version="1.1", en_callable=en_callable)

def check_exist(model_folder, name):
    path = os.path.join(model_folder, name)
    if not os.path.exists(path):
        print(f"Please download kokoro model file {name} to {model_folder}! Or set env var MD2VIDEO_KOKORO_MODEL_FOLDER.\n{DOWN_FILE}\n")
    return path

def text_to_speech(text, output_path, lang=None, speed=1.0):
    voice = "zf_001"
    model_folder = os.environ.get("MD2VIDEO_KOKORO_MODEL_FOLDER", kokoro, os.path.join(project_dir, "models"))
    onnx_path = check_exist(model_folder, "kokoro-v1.1-zh.onnx")
    bin_path = check_exist(model_folder, "voices-v1.1-zh.bin")
    vocab_config = check_exist(model_folder, "config.json")
    for p in [onnx_path, bin_path, vocab_config]:
        assert os.path.exists(p), "TTS model does not exists."
    kokoro = Kokoro(onnx_path, bin_path, vocab_config=vocab_config)
    phonemes, _ = g2p(text)
    samples, sample_rate = kokoro.create(phonemes, voice=voice, lang=lang, speed=speed, is_phonemes=True)
    sf.write(output_path, samples, sample_rate)
    assert os.path.exists(output_path)

# def text_to_speech(text, output_path, lang=None):
#     assert type(text) is str
#     if lang is None:
#         lang = os.environ.get("MD2VIDEO_LANG", 'cmn')
#     import tempfile
#     assert(output_path.endswith('.wav'))
    
#     temp_text = tempfile.NamedTemporaryFile(mode='w', suffix='.txt', delete=False)
#     temp_text_path = temp_text.name
#     try:
#         temp_text.write(text)
#         temp_text.close()  # Close file before subprocess uses it
        
#         # Run kokoro-tts command
#         kokoro_path = os.environ.get("MD2VIDEO_KOKORO", os.path.join(project_dir, 'kokoro-tts', 'kokoro-tts'))
#         voice = os.environ.get("MD2VIDEO_KOKORO_VOICE", default_voice_for_lang[lang])
#         assert kokoro_path is not None and os.path.exists(kokoro_path), "Please set path to kokoro-tts in MD2VIDEO_KOKORO"
#         sub = subprocess.run([sys.executable, kokoro_path, os.path.abspath(temp_text_path), os.path.abspath(output_path), '--voice', voice], 
#                       cwd=os.path.dirname(kokoro_path), check=True)
#         assert os.path.exists(output_path)
#     finally:
#         # Clean up temp file
#         if os.path.exists(temp_text_path):
#             os.unlink(temp_text_path)
